/*
* Jalview - A Sequence Alignment Editor and Viewer
* Copyright (C) 2006 AM Waterhouse, J Procter, G Barton, M Clamp, S Searle
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
*/
package jalview.datamodel;

import java.awt.*;

import java.util.*;


/**
 * DOCUMENT ME!
 *
 * @author $author$
 * @version $Revision: 1.39 $
 */
public class Sequence implements SequenceI
{
    SequenceI datasetSequence;
    String name;
    private String sequence;
    String description;
    int start;
    int end;
    Color color = Color.white;
    Vector pdbIds;
    String vamsasId;
    DBRefEntry [] dbrefs;

    /** This annotation is displayed below the alignment but the
     * positions are tied to the residues of this sequence */
    Vector annotation;

    /** DOCUMENT ME!! */
    public SequenceFeature [] sequenceFeatures;

    /** This array holds hidden sequences
     * of which this sequence is the representitive member of a group
     */
    SequenceGroup hiddenSequences;

    /**
     * Creates a new Sequence object.
     *
     * @param name DOCUMENT ME!
     * @param sequence DOCUMENT ME!
     * @param start DOCUMENT ME!
     * @param end DOCUMENT ME!
     */
    public Sequence(String name, String sequence, int start, int end)
    {
      this.name = name;
      this.sequence = sequence;
      this.start = start;
      this.end = end;

      parseId();

      checkValidRange();
    }

    com.stevesoft.pat.Regex limitrx = new com.stevesoft.pat.Regex(
                        "[/][0-9]{1,}[-][0-9]{1,}$");
    com.stevesoft.pat.Regex endrx = new com.stevesoft.pat.Regex(
                        "[0-9]{1,}$");

    void parseId()
    {
        // Does sequence have the /start-end signiature?
         if(limitrx.search(name))
         {
            name = limitrx.left();
            endrx.search(limitrx.stringMatched());
            setStart( Integer.parseInt( limitrx.stringMatched().substring(1,endrx.matchedFrom()-1 )));
            setEnd(   Integer.parseInt( endrx.stringMatched() ));
         }
    }

    void checkValidRange()
    {
      if (end < 1)
      {
        int endRes = 0;
        char ch;
        for (int j = 0; j < sequence.length(); j++)
        {
          ch = sequence.charAt(j);
          if (!jalview.util.Comparison.isGap( (ch)))
          {
            endRes++;
          }
        }
        if (endRes > 0)
        {
          endRes += start - 1;
        }

        this.end = endRes;
      }

    }

    /**
     * Creates a new Sequence object.
     *
     * @param name DOCUMENT ME!
     * @param sequence DOCUMENT ME!
     */
    public Sequence(String name, String sequence)
    {
        this(name, sequence, 1, -1);
    }

    /**
     * Creates a new Sequence object.
     *
     * @param seq DOCUMENT ME!
     */
    public Sequence(SequenceI seq)
    {
        this(seq.getName(), seq.getSequence(), seq.getStart(), seq.getEnd());
    }

    /**
     * DOCUMENT ME!
     *
     * @param v DOCUMENT ME!
     */
    public void setSequenceFeatures(SequenceFeature [] features)
    {
        sequenceFeatures = features;
    }

    public synchronized void addSequenceFeature(SequenceFeature sf)
    {
      if(sequenceFeatures==null)
      {
        sequenceFeatures = new SequenceFeature[0];
      }

      for(int i=0; i<sequenceFeatures.length; i++)
      {
        if(sequenceFeatures[i].equals(sf))
        {
          return;
        }
      }

      SequenceFeature [] temp = new SequenceFeature[sequenceFeatures.length+1];
      System.arraycopy(sequenceFeatures, 0, temp, 0, sequenceFeatures.length);
      temp[sequenceFeatures.length] = sf;

      sequenceFeatures = temp;
    }


    /**
     * DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public SequenceFeature [] getSequenceFeatures()
    {
        return sequenceFeatures;
    }

    public void addPDBId(PDBEntry entry)
    {
      if(pdbIds == null)
        pdbIds = new Vector();

      pdbIds.addElement(entry);
    }

    /**
     * DOCUMENT ME!
     *
     * @param id DOCUMENT ME!
     */
    public void setPDBId(Vector id)
    {
        pdbIds = id;
    }

    /**
     * DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public Vector getPDBId()
    {
        return pdbIds;
    }

    /**
     * DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public String getDisplayId(boolean jvsuffix)
    {
      StringBuffer result = new StringBuffer(name);
      if (jvsuffix)
      {
        result.append("/" + start + "-" + end);
      }

      return result.toString();
    }

    /**
     * DOCUMENT ME!
     *
     * @param name DOCUMENT ME!
     */
    public void setName(String name)
    {
      this.name = name;
      this.parseId();
    }

    /**
     * DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public String getName()
    {
       return this.name;
    }

    /**
     * DOCUMENT ME!
     *
     * @param start DOCUMENT ME!
     */
    public void setStart(int start)
    {
        this.start = start;
    }

    /**
     * DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public int getStart()
    {
        return this.start;
    }

    /**
     * DOCUMENT ME!
     *
     * @param end DOCUMENT ME!
     */
    public void setEnd(int end)
    {
        this.end = end;
    }

    /**
     * DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public int getEnd()
    {
        return this.end;
    }

    /**
     * DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public int getLength()
    {
        return this.sequence.length();
    }

    /**
     * DOCUMENT ME!
     *
     * @param seq DOCUMENT ME!
     */
    public void setSequence(String seq)
    {
        this.sequence = seq;
        checkValidRange();
    }

    /**
     * DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public String getSequence()
    {
        return this.sequence;
    }

    /**
     * DOCUMENT ME!
     *
     * @param start DOCUMENT ME!
     * @param end DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public String getSequence(int start, int end)
    {
        // JBPNote - left to user to pad the result here (TODO:Decide on this policy)
        if (start >= sequence.length())
        {
            return "";
        }

        if (end >= sequence.length())
        {
            end = sequence.length();
        }

        return this.sequence.substring(start, end);
    }
    /**
     * make a new Sequence object from start to end (including gaps) over this seqeunce
     * @param start int
     * @param end int
     * @return SequenceI
     */
    public SequenceI getSubSequence(int start, int end) {
      if (start<0)
        start = 0;
      String seq = getSequence(start, end);
      if (seq=="")
        return null;
      int nstart = findPosition(start);
      int nend=findPosition(end)-1;
      // JBPNote - this is an incomplete copy.
      SequenceI nseq = new Sequence(this.getName(), seq, nstart, nend);
      nseq.setDatasetSequence(getDatasetSequence());
      return nseq;
    }
    /**
     * DOCUMENT ME!
     *
     * @param i DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public char getCharAt(int i)
    {
        if (i < sequence.length())
        {
            return sequence.charAt(i);
        }
        else
        {
            return ' ';
        }
    }

    /**
     * DOCUMENT ME!
     *
     * @param desc DOCUMENT ME!
     */
    public void setDescription(String desc)
    {
        this.description = desc;
    }

    /**
     * DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public String getDescription()
    {
        return this.description;
    }

    /**
     * DOCUMENT ME!
     *
     * @param pos DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public int findIndex(int pos)
    {
        // returns the alignment position for a residue
        int j = start;
        int i = 0;

        while ((i < sequence.length()) && (j <= end) && (j <= pos))
        {
            if (!jalview.util.Comparison.isGap(sequence.charAt(i)))
            {
                j++;
            }

            i++;
        }

        if ((j == end) && (j < pos))
        {
            return end + 1;
        }
        else
        {
            return i;
        }
    }

    /**
     * Returns the sequence position for an alignment position
     *
     * @param i column index in alignment (from 1)
     *
     * @return residue number for residue (left of and) nearest ith column
     */
    public int findPosition(int i)
    {
        int j = 0;
        int pos = start;
        int seqlen=sequence.length();
        while ((j < i) && (j < seqlen))
        {
            if (!jalview.util.Comparison.isGap((sequence.charAt(j))))
            {
                pos++;
            }

            j++;
        }

        return pos;
    }

    /**
     * DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public int[] gapMap()
    {
        // Returns an int array giving the position of each residue in the sequence in the alignment
        String seq = jalview.analysis.AlignSeq.extractGaps(jalview.util.Comparison.GapChars, sequence);
        int[] map = new int[seq.length()];
        int j = 0;
        int p = 0;

        while (j < sequence.length())
        {
            if (!jalview.util.Comparison.isGap(sequence.charAt(j)))
            {
                map[p++] = j;
            }

            j++;
        }

        return map;
    }

    /**
     * DOCUMENT ME!
     *
     * @param i DOCUMENT ME!
     */
    public void deleteCharAt(int i)
    {
        if (i >= sequence.length())
        {
            return;
        }

        sequence = sequence.substring(0, i) + sequence.substring(i + 1);
    }

    /**
     * DOCUMENT ME!
     *
     * @param i DOCUMENT ME!
     * @param j DOCUMENT ME!
     */
    public void deleteChars(int i, int j)
    {
        if (i >= sequence.length())
        {
            return;
        }

        if (j >= sequence.length())
        {
            sequence = sequence.substring(0, i);
        }
        else
        {
            sequence = sequence.substring(0, i) + sequence.substring(j);
        }
    }


    /**
     * DOCUMENT ME!
     *
     * @param i DOCUMENT ME!
     * @param c DOCUMENT ME!
     * @param chop DOCUMENT ME!
     */
    public void insertCharAt(int i, char c)
    {
        String tmp = new String(sequence);

        if (i < sequence.length())
        {
            sequence = tmp.substring(0, i) + String.valueOf(c) +
                tmp.substring(i);
        }
        else
        {
            // JBPNote : padding char at end of sequence. We'll not get away with this when we insert residues, I bet!
            char[] ch = new char[(1 + i) - sequence.length()];

            for (int j = 0, k = ch.length; j < k; j++)
                ch[j] = c;

            sequence = tmp + String.valueOf(ch);
        }
    }

    /**
     * DOCUMENT ME!
     *
     * @param c DOCUMENT ME!
     */
    public void setColor(Color c)
    {
        this.color = c;
    }

    /**
     * DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public Color getColor()
    {
        return color;
    }

    public String getVamsasId()
    {
      return vamsasId;
    }

    public void setVamsasId(String id)
    {
      vamsasId = id;
    }

    public void setDBRef(DBRefEntry [] dbref)
    {
      dbrefs = dbref;
    }

    public DBRefEntry [] getDBRef()
    {
      return dbrefs;
    }

    public void addDBRef(DBRefEntry entry)
    {
      if(dbrefs == null)
        dbrefs = new DBRefEntry[0];

      DBRefEntry [] temp = new DBRefEntry[dbrefs.length+1];
      System.arraycopy(dbrefs, 0, temp, 0, dbrefs.length);

      temp[temp.length-1] = entry;

      dbrefs = temp;
    }

    public void setDatasetSequence(SequenceI seq)
    {
      datasetSequence = seq;
    }

    public SequenceI getDatasetSequence()
    {
      return datasetSequence;
    }

    public AlignmentAnnotation [] getAnnotation()
    {
      if(annotation==null)
        return null;

      AlignmentAnnotation [] ret = new AlignmentAnnotation[annotation.size()];
      for(int r = 0; r<ret.length; r++)
        ret[r] = (AlignmentAnnotation)annotation.elementAt(r);

      return ret;
    }

    public void addAlignmentAnnotation(AlignmentAnnotation annotation)
    {
      if(this.annotation==null)
        this.annotation = new Vector();

      this.annotation.addElement( annotation );
    }

    public SequenceGroup getHiddenSequences()
    {
      return hiddenSequences;
    }

    public void addHiddenSequence(SequenceI seq)
    {
      if(hiddenSequences==null)
      {
        hiddenSequences = new SequenceGroup();
      }
      hiddenSequences.addSequence(seq, false);
    }

    public void showHiddenSequence(SequenceI seq)
    {
      hiddenSequences.deleteSequence(seq, false);
      if (hiddenSequences.getSize(false) < 1)
      {
        hiddenSequences = null;
      }
    }

    public void changeCase(boolean toUpper, int start, int end)
    {
      StringBuffer newSeq = new StringBuffer();

      if(end>sequence.length())
        end = sequence.length();

      if (start > 0)
      {
        newSeq.append(sequence.substring(0, start));
      }

      if (toUpper)
        newSeq.append(sequence.substring(start, end).toUpperCase());
      else
        newSeq.append(sequence.substring(start, end).toLowerCase());

      if (end < sequence.length())
        newSeq.append(sequence.substring(end));

      sequence = newSeq.toString();
    }

    public void toggleCase(int start, int end)
    {
      StringBuffer newSeq = new StringBuffer();

     if(end>sequence.length())
       end = sequence.length();

     if (start > 0)
     {
       newSeq.append(sequence.substring(0, start));
     }

     char nextChar;
     for(int c=start; c<end; c++)
     {
       nextChar = sequence.charAt(c);
       if(Character.isLetter(nextChar))
       {
         if(Character.isUpperCase(nextChar))
           nextChar = Character.toLowerCase(nextChar);
         else
           nextChar = Character.toUpperCase(nextChar);
       }


       newSeq.append(nextChar);
     }

     if (end < sequence.length())
       newSeq.append(sequence.substring(end));

     sequence = newSeq.toString();
    }

  public SequenceI getSubSequence(int start)
  {
    int e=getLength();
    if (start>=e)
      return null;
    return getSubSequence(start, getLength());
  }

  public int removeGaps() {
    if (sequence!=null)
      return removeGaps(0, getLength());
    return 0;
  }

  public int removeGaps(int start, int end) {
    int jSize = getLength();
    int oSize=jSize;
    if (jSize<=start)
      return 0;
    if (end>jSize)
      end = jSize;

    // Removing a range is much quicker than removing gaps
    // one by one for long sequences
    int j = start;
    int rangeStart=-1, rangeEnd=-1;

    do
    {
      if (jalview.util.Comparison.isGap(getCharAt(j)))
      {
        if(rangeStart==-1)
         {
           rangeStart = j;
           rangeEnd = j+1;
         }
         else
         {
           rangeEnd++;
         }
         j++;
      }
      else
      {
        if(rangeStart>-1)
        {
          deleteChars(rangeStart, rangeEnd);
          j-=rangeEnd-rangeStart;
          jSize-=rangeEnd-rangeStart;
          rangeStart = -1;
          rangeEnd = -1;
        }
        else
          j++;
      }
    }
    while (j < end && j < jSize);
    if(rangeStart>-1)
    {
     deleteChars(rangeStart, rangeEnd);
     jSize-=rangeEnd-rangeStart;
    }
    return oSize-jSize; // number of deleted characters.
  }

}
