/*
 * Jalview - A Sequence Alignment Editor and Viewer
 * Copyright (C) 2006 AM Waterhouse, J Procter, G Barton, M Clamp, S Searle
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package jalview.gui;

import jalview.datamodel.*;

import java.awt.*;
import java.awt.image.*;

import javax.swing.*;


/**
 * DOCUMENT ME!
 *
 * @author $author$
 * @version $Revision: 1.41 $
 */
public class IdCanvas extends JPanel
{
    protected AlignViewport av;
    protected boolean showScores = true;
    protected int maxIdLength = -1;
    protected String maxIdStr = null;
    BufferedImage image;
    Graphics2D gg;
    int imgHeight = 0;
    boolean fastPaint = false;
    java.util.Vector searchResults;

    /**
     * Creates a new IdCanvas object.
     *
     * @param av DOCUMENT ME!
     */
    public IdCanvas(AlignViewport av)
    {
        setLayout(new BorderLayout());
        this.av = av;
        PaintRefresher.Register(this, av.alignment);
    }

    /**
     * DOCUMENT ME!
     *
     * @param gg DOCUMENT ME!
     * @param s DOCUMENT ME!
     * @param i DOCUMENT ME!
     * @param starty DOCUMENT ME!
     * @param ypos DOCUMENT ME!
     */
    public void drawIdString(Graphics2D gg, SequenceI s, int i, int starty, int ypos)
    {
        int charHeight = av.charHeight;

        if ((searchResults != null) && searchResults.contains(s))
        {
            gg.setColor(Color.black);
            gg.fillRect(0, ((i - starty) * charHeight) + ypos, getWidth(),
                charHeight);
            gg.setColor(Color.white);
        }
        else if ((av.getSelectionGroup() != null) &&
                av.getSelectionGroup().getSequences(false).contains(s))
        {
            gg.setColor(Color.lightGray);
            gg.fillRect(0, ((i - starty) * charHeight) + ypos, getWidth(),
                charHeight);
            gg.setColor(Color.white);
        }
        else
        {
            gg.setColor(s.getColor());
            gg.fillRect(0, ((i - starty) * charHeight) + ypos, getWidth(),
                charHeight);
            gg.setColor(Color.black);
        }


        gg.drawString( s.getDisplayId(av.getShowJVSuffix()),
                      0, (((i - starty + 1) * charHeight) + ypos) - (charHeight / 5));

        if (av.hasHiddenRows && av.showHiddenMarkers)
          drawMarker(i, starty, ypos);


    }

    /**
     * DOCUMENT ME!
     *
     * @param vertical DOCUMENT ME!
     */
    public void fastPaint(int vertical)
    {
        if (gg == null)
        {
            repaint();

            return;
        }

        gg.copyArea(0, 0, getWidth(), imgHeight, 0, -vertical * av.charHeight);

        int ss = av.startSeq;
        int es = av.endSeq;
        int transY = 0;

        if (vertical > 0) // scroll down
        {
            ss = es - vertical;

            if (ss < av.startSeq)
            { // ie scrolling too fast, more than a page at a time
                ss = av.startSeq;
            }
            else
            {
                transY = imgHeight - (vertical * av.charHeight);
            }
        }
        else if (vertical < 0)
        {
            es = ss - vertical;

            if (es > av.endSeq)
            {
                es = av.endSeq;
            }
        }

        gg.translate(0, transY);

        drawIds(ss, es);

        gg.translate(0, -transY);

        fastPaint = true;
        repaint();
    }

    /**
     * DOCUMENT ME!
     *
     * @param g DOCUMENT ME!
     */
    public void paintComponent(Graphics g)
    {
        g.setColor(Color.white);
        g.fillRect(0, 0, getWidth(), getHeight());

        if (fastPaint)
        {
            fastPaint = false;
            g.drawImage(image, 0, 0, this);

            return;
        }

        int oldHeight = imgHeight;

        imgHeight = getHeight();
        imgHeight -= (imgHeight % av.charHeight);

        if (imgHeight < 1)
        {
            return;
        }

        if(oldHeight!=imgHeight || image.getWidth(this)!=getWidth())
        {
          image = new BufferedImage(getWidth(), imgHeight,
                                    BufferedImage.TYPE_INT_RGB);
        }

        gg = (Graphics2D) image.getGraphics();
        //Fill in the background
        gg.setColor(Color.white);
        gg.fillRect(0, 0, getWidth(), imgHeight);

        drawIds(av.getStartSeq(), av.endSeq);

        g.drawImage(image, 0, 0, this);
    }

    /**
     * DOCUMENT ME!
     *
     * @param starty DOCUMENT ME!
     * @param endy DOCUMENT ME!
     */
    void drawIds(int starty, int endy)
    {
      Font italic = new Font(av.getFont().getName(), Font.ITALIC,
                             av.getFont().getSize());

      gg.setFont(italic);

      if (av.antiAlias)
        gg.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
                            RenderingHints.VALUE_ANTIALIAS_ON);

        Color currentColor = Color.white;
        Color currentTextColor = Color.black;

        if (av.getWrapAlignment())
        {
          int maxwidth = av.alignment.getWidth();
          int alheight = av.alignment.getHeight();

          if (av.hasHiddenColumns)
            maxwidth = av.getColumnSelection().findColumnPosition(maxwidth) - 1;

          int annotationHeight = 0;
          AnnotationLabels labels = null;

          if(av.showAnnotation)
          {
            AnnotationPanel ap = new AnnotationPanel(av);
            annotationHeight = ap.adjustPanelHeight();
            labels = new AnnotationLabels(av);
          }

          int hgap = av.charHeight;
          if (av.scaleAboveWrapped)
            hgap += av.charHeight;

          int cHeight = alheight * av.charHeight
              + hgap
              + annotationHeight;

          int rowSize = av.getEndRes() - av.getStartRes();


            // Draw the rest of the panels
            for (int ypos = hgap, row = av.startRes;
                    (ypos <= getHeight()) && (row < maxwidth);
                    ypos += cHeight, row += rowSize)
            {
              for (int i = starty; i < alheight; i++)
              {
                if (av.hasHiddenRows)
                {
                  setHiddenFont(i);
                }
                else
                  gg.setFont(italic);

                SequenceI s = av.alignment.getSequenceAt(i);
                drawIdString(gg, s, i, 0, ypos);
              }

                if(labels!=null)
                {
                  gg.translate(0, ypos+(alheight * av.charHeight));
                  labels.drawComponent(gg, getWidth());
                  gg.translate(0, -ypos-(alheight * av.charHeight));
                }


            }
        }
        else
        {
          //Now draw the id strings

            //Now draw the id strings
            for (int i = starty; i < endy; i++)
            {
              if (av.hasHiddenRows)
              {
                setHiddenFont(i);
              }

                // Selected sequence colours
                if ( (searchResults != null) &&
                    searchResults.contains(av.alignment.getSequenceAt(i)))
                {
                  currentColor = Color.black;
                  currentTextColor = Color.white;
                }
                else if ( (av.getSelectionGroup() != null) &&
                         av.getSelectionGroup().getSequences(false).contains(
                             av.alignment.getSequenceAt(i)))
                {
                  currentColor = Color.lightGray;
                  currentTextColor = Color.black;
                }
                else
                {
                  currentColor = av.alignment.getSequenceAt(i).getColor();
                  currentTextColor = Color.black;
                }

                gg.setColor(currentColor);

                gg.fillRect(0, (i - starty) * av.charHeight, getWidth(),
                            av.charHeight);

                gg.setColor(currentTextColor);

                String string = av.alignment.getSequenceAt(i).getDisplayId( av.getShowJVSuffix());

                gg.drawString(string, 0,
                    (((i - starty) * av.charHeight) + av.charHeight) -
                    (av.charHeight / 5));

               if(av.hasHiddenRows && av.showHiddenMarkers)
                 drawMarker(i, starty, 0);

            }

        }
    }

    void drawMarker(int i, int starty, int yoffset)
    {
      int hiddenIndex = av.adjustForHiddenSeqs(i);
      int lastIndex = av.adjustForHiddenSeqs(i - 1);
      int nextIndex = av.adjustForHiddenSeqs(i + 1);

      boolean below = (hiddenIndex > lastIndex + 1);
      boolean above = (nextIndex>hiddenIndex+1);

        gg.setColor(Color.blue);
        if(below)
        {
          gg.fillPolygon(new int[]
                         {getWidth()- av.charHeight,
                         getWidth()- av.charHeight,
                         getWidth()},
                         new int[]
                         {
                         (i - starty) * av.charHeight +yoffset,
                         (i - starty) * av.charHeight +yoffset+ av.charHeight / 4,
                         (i - starty) * av.charHeight+yoffset
          }, 3);
        }
        if(above)
        {
          gg.fillPolygon(new int[]
                        {getWidth()- av.charHeight,
                        getWidth()- av.charHeight,
                        getWidth() },
                        new int[]
                        {
                        (i - starty+1) * av.charHeight +yoffset,
                        (i - starty+1) * av.charHeight +yoffset- av.charHeight / 4,
                        (i - starty+1) * av.charHeight +yoffset
         }, 3);

        }
    }

    void setHiddenFont(int i)
    {
      Font italic = new Font(av.getFont().getName(), Font.ITALIC,
                             av.getFont().getSize());
      Font bold = new Font(av.getFont().getName(), Font.BOLD,
                           av.getFont().getSize());

      if (av.alignment.getSequenceAt(i).getHiddenSequences() != null)
        gg.setFont(bold);
      else
        gg.setFont(italic);
    }

    /**
     * DOCUMENT ME!
     *
     * @param found DOCUMENT ME!
     */
    public void setHighlighted(java.util.Vector found)
    {
        searchResults = found;
        repaint();
    }
}
