/*
 * Jalview - A Sequence Alignment Editor and Viewer
 * Copyright (C) 2006 AM Waterhouse, J Procter, G Barton, M Clamp, S Searle
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package jalview.gui;

import MCview.*;

import jalview.analysis.*;

import jalview.datamodel.*;


import jalview.schemes.*;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;

import java.util.Vector;
import jalview.io.FormatAdapter;


/**
 * DOCUMENT ME!
 *
 * @author $author$
 * @version $Revision: 1.59 $
 */
public class PopupMenu extends JPopupMenu
{
    JMenu groupMenu = new JMenu();
    JMenuItem groupName = new JMenuItem();
    protected JRadioButtonMenuItem clustalColour = new JRadioButtonMenuItem();
    protected JRadioButtonMenuItem zappoColour = new JRadioButtonMenuItem();
    protected JRadioButtonMenuItem taylorColour = new JRadioButtonMenuItem();
    protected JRadioButtonMenuItem hydrophobicityColour = new JRadioButtonMenuItem();
    protected JRadioButtonMenuItem helixColour = new JRadioButtonMenuItem();
    protected JRadioButtonMenuItem strandColour = new JRadioButtonMenuItem();
    protected JRadioButtonMenuItem turnColour = new JRadioButtonMenuItem();
    protected JRadioButtonMenuItem buriedColour = new JRadioButtonMenuItem();
    protected JCheckBoxMenuItem abovePIDColour = new JCheckBoxMenuItem();
    protected JRadioButtonMenuItem userDefinedColour = new JRadioButtonMenuItem();
    protected JRadioButtonMenuItem PIDColour = new JRadioButtonMenuItem();
    protected JRadioButtonMenuItem BLOSUM62Colour = new JRadioButtonMenuItem();
    JRadioButtonMenuItem noColourmenuItem = new JRadioButtonMenuItem();
    protected JCheckBoxMenuItem conservationMenuItem = new JCheckBoxMenuItem();
    AlignmentPanel ap;
    JMenu sequenceMenu = new JMenu();
    JMenuItem sequenceName = new JMenuItem();
    Sequence sequence;
    JMenuItem unGroupMenuItem = new JMenuItem();
    JMenuItem outline = new JMenuItem();
    JRadioButtonMenuItem nucleotideMenuItem = new JRadioButtonMenuItem();
    JMenu colourMenu = new JMenu();
    JCheckBoxMenuItem showBoxes = new JCheckBoxMenuItem();
    JCheckBoxMenuItem showText = new JCheckBoxMenuItem();
    JCheckBoxMenuItem showColourText = new JCheckBoxMenuItem();
  JMenu editMenu = new JMenu();
  JMenuItem cut = new JMenuItem();
  JMenuItem copy = new JMenuItem();
  JMenuItem upperCase = new JMenuItem();
  JMenuItem lowerCase = new JMenuItem();
  JMenuItem toggle = new JMenuItem();
  JMenu pdbMenu = new JMenu();
  JMenuItem pdbFromFile = new JMenuItem();
  JMenuItem enterPDB = new JMenuItem();
  JMenuItem discoverPDB = new JMenuItem();
  JMenu outputMenu = new JMenu();

  /**
     * Creates a new PopupMenu object.
     *
     * @param ap DOCUMENT ME!
     * @param seq DOCUMENT ME!
     */
    public PopupMenu(final AlignmentPanel ap, Sequence seq, Vector links)
    {
        ///////////////////////////////////////////////////////////
        // If this is activated from the sequence panel, the user may want to
        // edit or annotate a particular residue. Therefore display the residue menu
        //
        // If from the IDPanel, we must display the sequence menu
        //////////////////////////////////////////////////////////
        this.ap = ap;
        sequence = seq;

        ButtonGroup colours = new ButtonGroup();
        colours.add(noColourmenuItem);
        colours.add(clustalColour);
        colours.add(zappoColour);
        colours.add(taylorColour);
        colours.add(hydrophobicityColour);
        colours.add(helixColour);
        colours.add(strandColour);
        colours.add(turnColour);
        colours.add(buriedColour);
        colours.add(abovePIDColour);
        colours.add(userDefinedColour);
        colours.add(PIDColour);
        colours.add(BLOSUM62Colour);

        for (int i = 0; i < jalview.io.FormatAdapter.WRITEABLE_FORMATS.length; i++)
        {
          JMenuItem item = new JMenuItem( jalview.io.FormatAdapter.WRITEABLE_FORMATS[i] );

          item.addActionListener(new java.awt.event.ActionListener()
          {
            public void actionPerformed(ActionEvent e)
            {
              outputText_actionPerformed(e);
            }
          });

          outputMenu.add(item);
        }


        try
        {
            jbInit();
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }


        if (seq != null)
        {
          int start = Math.max(sequence.getName().length()-15, 0);
          sequenceMenu.setText(sequence.getName().substring(start));

          JMenuItem menuItem;
          if( seq.getDatasetSequence().getPDBId() != null)
          {
            java.util.Enumeration e = seq.getDatasetSequence().getPDBId().
                elements();

            while (e.hasMoreElements())
            {
              final PDBEntry pdb = (PDBEntry) e.nextElement();

              menuItem = new JMenuItem();
              menuItem.setText("View PDB entry: " + pdb.getId());
              menuItem.addActionListener(new java.awt.event.ActionListener()
              {
                public void actionPerformed(ActionEvent e)
                {
                  new PDBViewer(pdb, sequence, ap.seqPanel.seqCanvas);
                }
              });
              sequenceMenu.add(menuItem);
            }
          }

          menuItem = new JMenuItem("Hide Sequences");
          menuItem.addActionListener(new java.awt.event.ActionListener()
              {
                public void actionPerformed(ActionEvent e)
                {
                  hideSequences(false);
                }
              });
          add(menuItem);

          if(ap.av.getSelectionGroup() !=null
             && ap.av.getSelectionGroup().getSize(false)>1)
          {
            menuItem = new JMenuItem("Represent Group with " + seq.getName());
            menuItem.addActionListener(new java.awt.event.ActionListener()
            {
              public void actionPerformed(ActionEvent e)
              {
                hideSequences(true);
              }
            });
            sequenceMenu.add(menuItem);
          }

          if (ap.av.hasHiddenRows)
          {
            final int index = ap.av.alignment.findIndex(seq);

            if (ap.av.adjustForHiddenSeqs(index) -
                ap.av.adjustForHiddenSeqs(index - 1) > 1)
            {
              menuItem = new JMenuItem("Reveal Sequences");
              menuItem.addActionListener(new ActionListener()
              {
                public void actionPerformed(ActionEvent e)
                {
                  ap.av.showSequence(index);
                  if (ap.overviewPanel != null)
                    ap.overviewPanel.updateOverviewImage();
                }
              });
              add(menuItem);
            }

            menuItem = new JMenuItem("Reveal All");
            menuItem.addActionListener(new ActionListener()
                {
                  public void actionPerformed(ActionEvent e)
                  {
                    ap.av.showAllHiddenSeqs();
                    if (ap.overviewPanel != null)
                      ap.overviewPanel.updateOverviewImage();
                    }
                });

            add(menuItem);
          }


        }



        SequenceGroup sg = ap.av.getSelectionGroup();

        if (sg != null)
        {
            groupName.setText(sg.getName());

            if (sg.cs instanceof ZappoColourScheme)
            {
                zappoColour.setSelected(true);
            }
            else if (sg.cs instanceof TaylorColourScheme)
            {
                taylorColour.setSelected(true);
            }
            else if (sg.cs instanceof PIDColourScheme)
            {
                PIDColour.setSelected(true);
            }
            else if (sg.cs instanceof Blosum62ColourScheme)
            {
                BLOSUM62Colour.setSelected(true);
            }
            else if (sg.cs instanceof UserColourScheme)
            {
                userDefinedColour.setSelected(true);
            }
            else if (sg.cs instanceof HydrophobicColourScheme)
            {
                hydrophobicityColour.setSelected(true);
            }
            else if (sg.cs instanceof HelixColourScheme)
            {
                helixColour.setSelected(true);
            }
            else if (sg.cs instanceof StrandColourScheme)
            {
                strandColour.setSelected(true);
            }
            else if (sg.cs instanceof TurnColourScheme)
            {
                turnColour.setSelected(true);
            }
            else if (sg.cs instanceof BuriedColourScheme)
            {
                buriedColour.setSelected(true);
            }
            else if (sg.cs instanceof ClustalxColourScheme)
            {
                clustalColour.setSelected(true);
            }
            else
            {
                noColourmenuItem.setSelected(true);
            }

            if (sg.cs!=null && sg.cs.conservationApplied())
            {
                conservationMenuItem.setSelected(true);
            }

            showText.setSelected(sg.getDisplayText());
            showColourText.setSelected(sg.getColourText());
            showBoxes.setSelected(sg.getDisplayBoxes());
        }
        else
        {
          groupMenu.setVisible(false);
          editMenu.setVisible(false);
        }

        if (!ap.av.alignment.getGroups().contains(sg))
        {
            unGroupMenuItem.setVisible(false);
        }


        if (seq == null)
        {
            sequenceMenu.setVisible(false);
            pdbMenu.setVisible(false);
        }

        if(links != null && links.size()>0)
        {
          JMenu linkMenu = new JMenu("Link");
          JMenuItem item;
          for(int i=0; i<links.size(); i++)
          {
            String link = links.elementAt(i).toString();
            final String label = link.substring(0, link.indexOf("|"));
            item = new JMenuItem(label);
            final String url;

            if (link.indexOf("$SEQUENCE_ID$") > -1)
            {
              String id = seq.getName();
              if (id.indexOf("|") > -1)
                id = id.substring(id.lastIndexOf("|") + 1);

              url = link.substring(link.indexOf("|") + 1,
                                   link.indexOf("$SEQUENCE_ID$"))
                  + id +
                  link.substring(link.indexOf("$SEQUENCE_ID$") + 13);
            }
            else
              url = link.substring(link.lastIndexOf("|")+1);


            item.addActionListener(new java.awt.event.ActionListener()
            {
                public void actionPerformed(ActionEvent e)
                {
                    showLink(url);
                }
            });

            linkMenu.add(item);
          }
          if(sequence!=null)
            sequenceMenu.add(linkMenu);
          else
            add(linkMenu);
        }
    }


    /**
     * DOCUMENT ME!
     *
     * @throws Exception DOCUMENT ME!
     */
    private void jbInit() throws Exception
    {
        groupMenu.setText("Group");
        groupMenu.setText("Selection");
        groupName.setText("Name");
        groupName.addActionListener(new java.awt.event.ActionListener()
            {
                public void actionPerformed(ActionEvent e)
                {
                    groupName_actionPerformed(e);
                }
            });
        sequenceMenu.setText("Sequence");
        sequenceName.setText("Edit Name/Description");
        sequenceName.addActionListener(new java.awt.event.ActionListener()
            {
                public void actionPerformed(ActionEvent e)
                {
                    sequenceName_actionPerformed(e);
                }
            });
        PIDColour.setFocusPainted(false);
        unGroupMenuItem.setText("Remove Group");
        unGroupMenuItem.addActionListener(new java.awt.event.ActionListener()
            {
                public void actionPerformed(ActionEvent e)
                {
                    unGroupMenuItem_actionPerformed(e);
                }
            });

        outline.setText("Border colour");
        outline.addActionListener(new java.awt.event.ActionListener()
            {
                public void actionPerformed(ActionEvent e)
                {
                    outline_actionPerformed(e);
                }
            });
        nucleotideMenuItem.setText("Nucleotide");
        nucleotideMenuItem.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent e)
                {
                    nucleotideMenuItem_actionPerformed(e);
                }
            });
        colourMenu.setText("Group Colour");
        showBoxes.setText("Boxes");
        showBoxes.setState(true);
        showBoxes.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent e)
                {
                    showBoxes_actionPerformed(e);
                }
            });
        showText.setText("Text");
        showText.setState(true);
        showText.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent e)
                {
                    showText_actionPerformed(e);
                }
            });
        showColourText.setText("Colour Text");
        showColourText.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent e)
                {
                    showColourText_actionPerformed(e);
                }
            });
    editMenu.setText("Edit");
    cut.setText("Cut");
    cut.addActionListener(new ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        cut_actionPerformed(e);
      }
    });
    upperCase.setText("To Upper Case");
    upperCase.addActionListener(new ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        upperCase_actionPerformed(e);
      }
    });
    copy.setText("Copy");
    copy.addActionListener(new ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        copy_actionPerformed(e);
      }
    });
    lowerCase.setText("To Lower Case");
    lowerCase.addActionListener(new ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        lowerCase_actionPerformed(e);
      }
    });
    toggle.setText("Toggle Case");
    toggle.addActionListener(new ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        toggle_actionPerformed(e);
      }
    });
    pdbMenu.setText("Associate Structure with Sequence");
    pdbFromFile.setText("From File");
    pdbFromFile.addActionListener(new ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        pdbFromFile_actionPerformed(e);
      }
    });
    enterPDB.setText("Enter PDB Id");
    enterPDB.addActionListener(new ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        enterPDB_actionPerformed(e);
      }
    });
    discoverPDB.setText("Discover PDB ids");
    discoverPDB.addActionListener(new ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        discoverPDB_actionPerformed(e);
      }
    });
    outputMenu.setText("Output to Textbox...");
    add(groupMenu);

    add(sequenceMenu);
    groupMenu.add(editMenu);
    groupMenu.add(outputMenu);
    groupMenu.addSeparator();
    groupMenu.add(groupName);
    groupMenu.add(unGroupMenuItem);
        groupMenu.add(colourMenu);
    groupMenu.add(showBoxes);
        groupMenu.add(showText);
        groupMenu.add(showColourText);
    groupMenu.add(outline);
        sequenceMenu.add(sequenceName);
    sequenceMenu.add(pdbMenu);
    colourMenu.add(noColourmenuItem);
        colourMenu.add(clustalColour);
        colourMenu.add(BLOSUM62Colour);
        colourMenu.add(PIDColour);
        colourMenu.add(zappoColour);
        colourMenu.add(taylorColour);
        colourMenu.add(hydrophobicityColour);
        colourMenu.add(helixColour);
        colourMenu.add(strandColour);
        colourMenu.add(turnColour);
        colourMenu.add(buriedColour);
        colourMenu.add(nucleotideMenuItem);
        colourMenu.add(userDefinedColour);

        if(jalview.gui.UserDefinedColours.getUserColourSchemes()!=null)
        {
          java.util.Enumeration userColours = jalview.gui.UserDefinedColours.
              getUserColourSchemes().keys();

          while (userColours.hasMoreElements())
          {
            JMenuItem item = new JMenuItem(userColours.
                nextElement().toString());
            item.addActionListener(new ActionListener()
            {
              public void actionPerformed(ActionEvent evt)
              {
                userDefinedColour_actionPerformed(evt);
              }
            });
            colourMenu.add(item);
          }
        }


        colourMenu.addSeparator();
        colourMenu.add(abovePIDColour);
        colourMenu.add(conservationMenuItem);
    editMenu.add(copy);
    editMenu.add(cut);
    editMenu.add(upperCase);
    editMenu.add(lowerCase);
    editMenu.add(toggle);
    pdbMenu.add(pdbFromFile);
    pdbMenu.add(enterPDB);
    pdbMenu.add(discoverPDB);
    noColourmenuItem.setText("None");
        noColourmenuItem.addActionListener(new java.awt.event.ActionListener()
            {
                public void actionPerformed(ActionEvent e)
                {
                    noColourmenuItem_actionPerformed(e);
                }
            });

        clustalColour.setText("Clustalx colours");
        clustalColour.addActionListener(new java.awt.event.ActionListener()
            {
                public void actionPerformed(ActionEvent e)
                {
                    clustalColour_actionPerformed(e);
                }
            });
        zappoColour.setText("Zappo");
        zappoColour.addActionListener(new java.awt.event.ActionListener()
            {
                public void actionPerformed(ActionEvent e)
                {
                    zappoColour_actionPerformed(e);
                }
            });
        taylorColour.setText("Taylor");
        taylorColour.addActionListener(new java.awt.event.ActionListener()
            {
                public void actionPerformed(ActionEvent e)
                {
                    taylorColour_actionPerformed(e);
                }
            });
        hydrophobicityColour.setText("Hydrophobicity");
        hydrophobicityColour.addActionListener(new java.awt.event.ActionListener()
            {
                public void actionPerformed(ActionEvent e)
                {
                    hydrophobicityColour_actionPerformed(e);
                }
            });
        helixColour.setText("Helix propensity");
        helixColour.addActionListener(new java.awt.event.ActionListener()
            {
                public void actionPerformed(ActionEvent e)
                {
                    helixColour_actionPerformed(e);
                }
            });
        strandColour.setText("Strand propensity");
        strandColour.addActionListener(new java.awt.event.ActionListener()
            {
                public void actionPerformed(ActionEvent e)
                {
                    strandColour_actionPerformed(e);
                }
            });
        turnColour.setText("Turn propensity");
        turnColour.addActionListener(new java.awt.event.ActionListener()
            {
                public void actionPerformed(ActionEvent e)
                {
                    turnColour_actionPerformed(e);
                }
            });
        buriedColour.setText("Buried Index");
        buriedColour.addActionListener(new java.awt.event.ActionListener()
            {
                public void actionPerformed(ActionEvent e)
                {
                    buriedColour_actionPerformed(e);
                }
            });
        abovePIDColour.setText("Above % Identity");
        abovePIDColour.addActionListener(new java.awt.event.ActionListener()
            {
                public void actionPerformed(ActionEvent e)
                {
                    abovePIDColour_actionPerformed(e);
                }
            });
        userDefinedColour.setText("User Defined...");
        userDefinedColour.addActionListener(new java.awt.event.ActionListener()
            {
                public void actionPerformed(ActionEvent e)
                {
                    userDefinedColour_actionPerformed(e);
                }
            });
        PIDColour.setText("Percentage Identity");
        PIDColour.addActionListener(new java.awt.event.ActionListener()
            {
                public void actionPerformed(ActionEvent e)
                {
                    PIDColour_actionPerformed(e);
                }
            });
        BLOSUM62Colour.setText("BLOSUM62");
        BLOSUM62Colour.addActionListener(new java.awt.event.ActionListener()
            {
                public void actionPerformed(ActionEvent e)
                {
                    BLOSUM62Colour_actionPerformed(e);
                }
            });
        conservationMenuItem.setText("Conservation");
        conservationMenuItem.addActionListener(new java.awt.event.ActionListener()
            {
                public void actionPerformed(ActionEvent e)
                {
                    conservationMenuItem_actionPerformed(e);
                }
            });
    }

    /**
     * DOCUMENT ME!
     */
    void refresh()
    {
        if (ap.overviewPanel != null)
          ap.overviewPanel.updateOverviewImage();

        ap.seqPanel.repaint();
    }

    /**
     * DOCUMENT ME!
     *
     * @param e DOCUMENT ME!
     */
    protected void clustalColour_actionPerformed(ActionEvent e)
    {
        SequenceGroup sg = getGroup();
        sg.cs = new ClustalxColourScheme(sg.getSequences(true),
                ap.av.alignment.getWidth());
        refresh();
    }

    /**
     * DOCUMENT ME!
     *
     * @param e DOCUMENT ME!
     */
    protected void zappoColour_actionPerformed(ActionEvent e)
    {
        getGroup().cs = new ZappoColourScheme();
        refresh();
    }

    /**
     * DOCUMENT ME!
     *
     * @param e DOCUMENT ME!
     */
    protected void taylorColour_actionPerformed(ActionEvent e)
    {
        getGroup().cs = new TaylorColourScheme();
        refresh();
    }

    /**
     * DOCUMENT ME!
     *
     * @param e DOCUMENT ME!
     */
    protected void hydrophobicityColour_actionPerformed(ActionEvent e)
    {
        getGroup().cs = new HydrophobicColourScheme();
        refresh();
    }

    /**
     * DOCUMENT ME!
     *
     * @param e DOCUMENT ME!
     */
    protected void helixColour_actionPerformed(ActionEvent e)
    {
        getGroup().cs = new HelixColourScheme();
        refresh();
    }

    /**
     * DOCUMENT ME!
     *
     * @param e DOCUMENT ME!
     */
    protected void strandColour_actionPerformed(ActionEvent e)
    {
        getGroup().cs = new StrandColourScheme();
        refresh();
    }

    /**
     * DOCUMENT ME!
     *
     * @param e DOCUMENT ME!
     */
    protected void turnColour_actionPerformed(ActionEvent e)
    {
        getGroup().cs = new TurnColourScheme();
        refresh();
    }

    /**
     * DOCUMENT ME!
     *
     * @param e DOCUMENT ME!
     */
    protected void buriedColour_actionPerformed(ActionEvent e)
    {
        getGroup().cs = new BuriedColourScheme();
        refresh();
    }

    /**
     * DOCUMENT ME!
     *
     * @param e DOCUMENT ME!
     */
    public void nucleotideMenuItem_actionPerformed(ActionEvent e)
    {
        getGroup().cs = new NucleotideColourScheme();
        refresh();
    }

    /**
     * DOCUMENT ME!
     *
     * @param e DOCUMENT ME!
     */
    protected void abovePIDColour_actionPerformed(ActionEvent e)
    {
        SequenceGroup sg = getGroup();
        if(sg.cs==null)
          return;

        if (abovePIDColour.isSelected())
        {
            sg.cs.setConsensus(AAFrequency.calculate(
                    sg.getSequences(true), 0,
                    ap.av.alignment.getWidth()));

            int threshold = SliderPanel.setPIDSliderSource(ap, sg.cs,
                    getGroup().getName());

           sg.cs.setThreshold(threshold, ap.av.getIgnoreGapsConsensus());

            SliderPanel.showPIDSlider();
        }
        else // remove PIDColouring
        {
            sg.cs.setThreshold(0, ap.av.getIgnoreGapsConsensus());
        }

        refresh();
    }

    /**
     * DOCUMENT ME!
     *
     * @param e DOCUMENT ME!
     */
    protected void userDefinedColour_actionPerformed(ActionEvent e)
    {
       SequenceGroup sg = getGroup();

       if (e.getActionCommand().equals("User Defined..."))
         new UserDefinedColours(ap, sg);
       else
       {
         UserColourScheme udc = (UserColourScheme) UserDefinedColours.
             getUserColourSchemes().get(e.getActionCommand());

         sg.cs = udc;
       }
     }
    /**
     * DOCUMENT ME!
     *
     * @param e DOCUMENT ME!
     */
    protected void PIDColour_actionPerformed(ActionEvent e)
    {
        SequenceGroup sg = getGroup();
        sg.cs = new PIDColourScheme();
        sg.cs.setConsensus(AAFrequency.calculate(sg.getSequences(true), 0,
                ap.av.alignment.getWidth()));
        refresh();
    }

    /**
     * DOCUMENT ME!
     *
     * @param e DOCUMENT ME!
     */
    protected void BLOSUM62Colour_actionPerformed(ActionEvent e)
    {
        SequenceGroup sg = getGroup();

        sg.cs = new Blosum62ColourScheme();

        sg.cs.setConsensus(AAFrequency.calculate(sg.getSequences(true), 0,
                ap.av.alignment.getWidth()));

        refresh();
    }

    /**
     * DOCUMENT ME!
     *
     * @param e DOCUMENT ME!
     */
    protected void noColourmenuItem_actionPerformed(ActionEvent e)
    {
        getGroup().cs = null;
        refresh();
    }

    /**
     * DOCUMENT ME!
     *
     * @param e DOCUMENT ME!
     */
    protected void conservationMenuItem_actionPerformed(ActionEvent e)
    {
        SequenceGroup sg = getGroup();
        if(sg.cs==null)
          return;

        if (conservationMenuItem.isSelected())
        {
            Conservation c = new Conservation("Group",
                    ResidueProperties.propHash, 3,
                    sg.getSequences(true), 0,
                    ap.av.alignment.getWidth());

            c.calculate();
            c.verdict(false, ap.av.ConsPercGaps);

            sg.cs.setConservation(c);

            SliderPanel.setConservationSlider(ap, sg.cs, sg.getName());
            SliderPanel.showConservationSlider();
        }
        else // remove ConservationColouring
        {
            sg.cs.setConservation(null);
        }

        refresh();
    }

    /**
     * DOCUMENT ME!
     *
     * @param e DOCUMENT ME!
     */
    protected void groupName_actionPerformed(ActionEvent e)
    {
        SequenceGroup sg = getGroup();
        String reply = JOptionPane.showInternalInputDialog(Desktop.desktop,
                "Enter new group name", "Edit group name",
                JOptionPane.QUESTION_MESSAGE);

        if (reply == null)
        {
            return;
        }

        sg.setName(reply);
        groupName.setText(reply);
    }

    /**
     * DOCUMENT ME!
     *
     * @param e DOCUMENT ME!
     */
    protected void analyze_actionPerformed(ActionEvent e)
    {
        CutAndPasteTransfer cap = new CutAndPasteTransfer();
        JInternalFrame frame = new JInternalFrame();
        frame.setContentPane(cap);
        Desktop.addInternalFrame(frame, "Analyze this - ", 400, 300);

        SequenceGroup sg = getGroup();
        StringBuffer sb = new StringBuffer();

        for (int i = 0; i < sg.getSize(false); i++)
        {
            Sequence tmp = (Sequence) sg.getSequences(false).elementAt(i);
            sb.append(tmp.getSequence(sg.getStartRes(), sg.getEndRes() + 1));
            sb.append("\n");
        }

        sb.append("Something amazing will happen soon");
        cap.setText(sb.toString());
    }

    /**
     * DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    SequenceGroup getGroup()
    {
        SequenceGroup sg = ap.av.getSelectionGroup();
      // this method won't add a new group if it already exists
        if(sg!=null)
          ap.av.alignment.addGroup(sg);

        return sg;
    }

    /**
     * DOCUMENT ME!
     *
     * @param e DOCUMENT ME!
     */
    void sequenceName_actionPerformed(ActionEvent e)
    {
      JLabel idlabel = new JLabel(  "       Sequence Name ");
      JLabel desclabel = new JLabel("Sequence Description ");
      idlabel.setFont(new Font("Courier", Font.PLAIN, 12));
      desclabel.setFont(new Font("Courier", Font.PLAIN, 12));
      JTextField id = new JTextField(sequence.getName(), 40);
      JTextField description = new JTextField(sequence.getDescription(), 40);
      JPanel panel = new JPanel(new BorderLayout());
      JPanel panel2 = new JPanel(new BorderLayout());
      panel2.add(idlabel, BorderLayout.WEST);
      panel2.add(id, BorderLayout.CENTER);
      panel.add(panel2, BorderLayout.NORTH);
      panel2 = new JPanel(new BorderLayout());
      panel2.add(desclabel, BorderLayout.WEST);
      panel2.add(description, BorderLayout.CENTER);
      panel.add(panel2, BorderLayout.SOUTH);


       int reply = JOptionPane.showInternalConfirmDialog(Desktop.desktop,
          panel, "Edit Sequence Name/Description",
          JOptionPane.OK_CANCEL_OPTION );


        if (reply != JOptionPane.OK_OPTION )
        {
            return;
        }

        String s = id.getText();

        if (s != null)
        {
            if (s.indexOf(" ") > -1)
            {
                JOptionPane.showMessageDialog(ap,
                    "Spaces have been converted to \"_\"",
                    "No spaces allowed in Sequence Name",
                    JOptionPane.WARNING_MESSAGE);
            }

            s = s.replace(' ', '_');
            sequence.getDatasetSequence().setName(s);
            sequence.setName(s);
            ap.repaint();
        }

        sequence.getDatasetSequence().setDescription(description.getText());
        sequence.setDescription(description.getText());
    }

    /**
     * DOCUMENT ME!
     *
     * @param e DOCUMENT ME!
     */
    void unGroupMenuItem_actionPerformed(ActionEvent e)
    {
        SequenceGroup sg = ap.av.getSelectionGroup();
        ap.av.alignment.deleteGroup(sg);
        ap.av.setSelectionGroup(null);
        refresh();
    }


    /**
     * DOCUMENT ME!
     *
     * @param e DOCUMENT ME!
     */
    protected void outline_actionPerformed(ActionEvent e)
    {
        SequenceGroup sg = getGroup();
        Color col = JColorChooser.showDialog(this, "Select Outline Colour",
                Color.BLUE);

        if (col != null)
        {
            sg.setOutlineColour(col);
        }

        refresh();
    }

    /**
     * DOCUMENT ME!
     *
     * @param e DOCUMENT ME!
     */
    public void showBoxes_actionPerformed(ActionEvent e)
    {
        getGroup().setDisplayBoxes(showBoxes.isSelected());
        refresh();
    }

    /**
     * DOCUMENT ME!
     *
     * @param e DOCUMENT ME!
     */
    public void showText_actionPerformed(ActionEvent e)
    {
        getGroup().setDisplayText(showText.isSelected());
        refresh();
    }

    /**
     * DOCUMENT ME!
     *
     * @param e DOCUMENT ME!
     */
    public void showColourText_actionPerformed(ActionEvent e)
    {
        getGroup().setColourText(showColourText.isSelected());
        refresh();
    }

    public void showLink(String url)
    {
      try
      {
        jalview.util.BrowserLauncher.openURL(url);
      }
      catch (Exception ex)
      {
        JOptionPane.showInternalMessageDialog(Desktop.desktop,
            "Unixers: Couldn't find default web browser."
           +"\nAdd the full path to your browser in Preferences.",
           "Web browser not found", JOptionPane.WARNING_MESSAGE );


        ex.printStackTrace();
      }
    }

    void hideSequences(boolean representGroup)
    {
      SequenceGroup sg = ap.av.getSelectionGroup();
      if(sg==null || sg.getSize(false)<1)
      {
        ap.av.hideSequence(sequence);
        return;
      }

        int index = 0;
        while(index < sg.getSize(false))
        {
          if(representGroup && sg.getSequenceAt(index)!=sequence)
          {
            sequence.addHiddenSequence(sg.getSequenceAt(index));
            ap.av.hideSequence(sg.getSequenceAt(index));
          }
          else if(!representGroup)
          {
            ap.av.hideSequence(sg.getSequenceAt(index));
          }
          index ++;
        }

        ap.av.setSelectionGroup(null);
    }

  public void copy_actionPerformed(ActionEvent e)
  {
    ap.alignFrame.copy_actionPerformed(null);
  }

  public void cut_actionPerformed(ActionEvent e)
  {
    ap.alignFrame.cut_actionPerformed(null);
  }

  public void upperCase_actionPerformed(ActionEvent e)
  {
    changeCase(e.getSource());
  }

  public void lowerCase_actionPerformed(ActionEvent e)
  {
    changeCase(e.getSource());
  }

  public void toggle_actionPerformed(ActionEvent e)
  {
    changeCase(e.getSource());
  }

  void changeCase(Object source)
  {
    SequenceGroup sg = ap.av.getSelectionGroup();
    if (sg != null)
    {
      for (int g = 0; g < sg.getSize(true); g++)
      {
        int start = sg.getStartRes();
        int end = sg.getEndRes() + 1;

        do
        {
          if (ap.av.hasHiddenColumns)
          {
            end = ap.av.colSel.getHiddenBoundaryRight(start);
            if (start == end)
              end = sg.getEndRes() + 1;
            if (end > sg.getEndRes())
              end = sg.getEndRes() + 1;
          }

          if (source == toggle)
            ( (SequenceI) sg.getSequences(true).elementAt(g))
                .toggleCase(start, end);
          else
            ( (SequenceI) sg.getSequences(true).elementAt(g))
                .changeCase(source == upperCase, start, end);

          if (ap.av.hasHiddenColumns)
          {
            start = ap.av.colSel.adjustForHiddenColumns(end);
            start = ap.av.colSel.getHiddenBoundaryLeft(start) + 1;
          }

        }
        while (end < sg.getEndRes());
      }
      ap.repaint();
    }
  }

  public void outputText_actionPerformed(ActionEvent e)
  {
    CutAndPasteTransfer cap = new CutAndPasteTransfer();
    Desktop.addInternalFrame(cap,
                             "Alignment output - " + e.getActionCommand(), 600,
                             500);

    String [] omitHidden = null;

    if(ap.av.hasHiddenColumns)
    {
      System.out.println("PROMPT USER HERE");
      omitHidden = ap.av.getViewAsString(true);
    }

    cap.setText(new FormatAdapter().formatSequences(
        e.getActionCommand(),
        ap.av.getSelectionAsNewSequence(),
        omitHidden));
  }


  public void pdbFromFile_actionPerformed(ActionEvent e)
  {
     jalview.io.JalviewFileChooser chooser
         = new jalview.io.JalviewFileChooser(jalview.bin.Cache.
         getProperty(
             "LAST_DIRECTORY"));
     chooser.setFileView(new jalview.io.JalviewFileView());
     chooser.setDialogTitle("Select a PDB file");
     chooser.setToolTipText("Load a PDB file");

     int value = chooser.showOpenDialog(null);

     if (value == jalview.io.JalviewFileChooser.APPROVE_OPTION)
     {
       PDBEntry entry = new PDBEntry();
       String choice = chooser.getSelectedFile().getPath();
       jalview.bin.Cache.setProperty("LAST_DIRECTORY", choice);
       try
       {
         MCview.PDBfile pdbfile = new MCview.PDBfile(choice,
                                       jalview.io.AppletFormatAdapter.FILE);

         if (pdbfile.id == null)
         {
           String reply = JOptionPane.showInternalInputDialog(
               Desktop.desktop,
               "Couldn't find a PDB id in the file supplied."
               + "Please enter an Id to identify this structure.",
               "No PDB Id in File", JOptionPane.QUESTION_MESSAGE);
           if (reply == null)
             return;

           entry.setId(reply);
         }
         else
           entry.setId(pdbfile.id);
       }
       catch (java.io.IOException ex)
       {
         ex.printStackTrace();
       }

       entry.setFile(choice);
       sequence.getDatasetSequence().addPDBId(entry);
     }

  }

  public void enterPDB_actionPerformed(ActionEvent e)
  {
    String id = JOptionPane.showInternalInputDialog(Desktop.desktop,
        "Enter PDB Id", "Enter PDB Id", JOptionPane.QUESTION_MESSAGE);

    if (id != null && id.length() > 0)
    {
      PDBEntry entry = new PDBEntry();
      entry.setId(id);
      sequence.getDatasetSequence()
          .addPDBId(entry);
    }
  }

  public void discoverPDB_actionPerformed(ActionEvent e)
  {
    new jalview.io.DBRefFetcher(
             ap.av.getAlignment(), ap.alignFrame).fetchDBRefs(false);
  }
}
