/*
* Jalview - A Sequence Alignment Editor and Viewer
* Copyright (C) 2006 AM Waterhouse, J Procter, G Barton, M Clamp, S Searle
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
*/
package jalview.io;

import jalview.datamodel.*;

import java.io.*;

import java.util.*;


/**
 * DOCUMENT ME!
 *
 * @author $author$
 * @version $Revision: 1.15 $
 */
public class BLCFile extends AlignFile
{
    Vector titles;

    /**
     * Creates a new BLCFile object.
     */
    public BLCFile()
    {
    }


    /**
     * Creates a new BLCFile object.
     *
     * @param inFile DOCUMENT ME!
     * @param type DOCUMENT ME!
     *
     * @throws IOException DOCUMENT ME!
     */
    public BLCFile(String inFile, String type) throws IOException
    {
        super(inFile, type);
    }

    /**
     * DOCUMENT ME!
     */
    public void initData()
    {
        super.initData();
        titles = new Vector();
    }

    /**
     * DOCUMENT ME!
     */
    public void parse() throws IOException
    {
        boolean idsFound = false;
        Vector ids = new Vector();
        StringBuffer[] seqstrings;
        Vector starts = new Vector();
        Vector ends = new Vector();

        String line = null;

            do
            {
                line = nextLine();

                // seek end of ids
                if (line.indexOf("*") > -1)
                {
                    idsFound = true;

                    break;
                }

                int abracket = line.indexOf(">");

                if (abracket > -1)
                {
                  if (line.indexOf(" ") > -1) //
                  {
                    line = line.substring(abracket + 1,
                                          line.indexOf(" ", abracket + 1));
                  }
                  else
                    line = line.substring(abracket+1);


                  Sequence seq = parseId(line);
                  ids.addElement(seq.getName());
                  starts.addElement(seq.getStart() + "");
                  ends.addElement(seq.getEnd() + "");
                }
            }
            while (!idsFound);

            int starCol = line.indexOf("*");
            seqstrings = new StringBuffer[ids.size()];

            for (int i = 0; i < ids.size(); i++)
            {
                if (seqstrings[i] == null)
                {
                    seqstrings[i] = new StringBuffer();
                }
            }

            while ((line = nextLine()).indexOf("*") == -1)
            {
                for (int i = 0; i < ids.size(); i++)
                {
                    if (line.length() > (i + starCol))
                    {
                        seqstrings[i].append(line.charAt(i + starCol));
                    }
                }
            }

            for (int i = 0; i < ids.size(); i++)
            {
              Sequence newSeq = new Sequence(ids.elementAt(i).toString(),
                                             seqstrings[i].toString(),
                                             Integer.parseInt(starts.elementAt(i).
                                                              toString()),
                                             Integer.parseInt(ends.elementAt(i).toString()));

              if (!isValidProteinSequence(newSeq.getSequence()))
              {
                throw new IOException(AppletFormatAdapter.INVALID_CHARACTERS
                               +" : "+ newSeq.getName()
                               +" : "+invalidCharacter);

              }

                seqs.addElement(newSeq);
            }

    }

    /**
     * DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public String print()
    {
        return print(getSeqsAsArray());
    }

    /**
     * DOCUMENT ME!
     *
     * @param s DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public String print(SequenceI[] s)
    {
        StringBuffer out = new StringBuffer();
        /**
         * A general parser for ids. Will look for dbrefs in
         * Uniprot format source|id
         * And also Jalview /start-end
         *
         * @String id Id to be parsed
     */
        int i = 0;
        int max = -1;

        while ((i < s.length) && (s[i] != null))
        {
            out.append(">" + printId(s[i]) +"\n");

            if (s[i].getSequence().length() > max)
            {
                max = s[i].getSequence().length();
            }

            i++;
        }

        out.append("* iteration 1\n");

        for (int j = 0; j < max; j++)
        {
            i = 0;

            while ((i < s.length) && (s[i] != null))
            {
                if (s[i].getSequence().length() > j)
                {
                    out.append(s[i].getSequence().substring(j, j + 1));
                }
                else
                {
                    out.append("-");
                }

                i++;
            }

            out.append("\n");
        }

        out.append("*\n");

        return out.toString();
    }
}
